import os, sys


# Link: https://github.com/spatialaudio/python-sounddevice/issues/11#issuecomment-155836787
'''
It seems to work by running this before each PortAudio call:

devnull = os.open(os.devnull, os.O_WRONLY)
old_stderr = os.dup(2)
sys.stderr.flush()
os.dup2(devnull, 2)
os.close(devnull)

and this immediately afterwards (ideally, in a with statement):

os.dup2(old_stderr, 2)
os.close(old_stderr)
'''


old_stderr = os.dup(2)
def mute_on():
    devnull = os.open(os.devnull, os.O_WRONLY)
    
    sys.stderr.flush()
    os.dup2(devnull, 2)
    os.close(devnull)

def mute_off():
    os.dup2(old_stderr, 2)
    os.close(old_stderr)


from .voice_recognition import voice
from importlib import resources
from .search import search
from .playmedia import play
import sys
from .imports import ascii_art
''' 
 to generate .exe file
 pyinstaller dist.py --add-data "youtube_stream/*.txt;youtube_stream" --onefile --name youtube_stream
 or pyinstaller youtube_stream.spec
 
'''
def run():
    print("\n\n")
    try:
        print(ascii_art)
    except:
        print("             Youtube-Stream      ")
    print("\n\n\n")
    while True:
        print("Type Song name (Use comma (,) for multiple songs)")
        print("Or Press ENTER for Voice Recognition. Say 'then', or 'plus' for multiple songs")
        print("Song Name: ",end ="")

        try:
            music_name = input()
        except KeyboardInterrupt:
            print('\nClose VLC to exit this program.')
            sys.exit(0)

        if len(music_name)==0:
            music_name = voice()
        if music_name is not None:
            videos = search(music_name)
            play(videos)
        print("Started Playing...\n\n")


import subprocess, os
from configparser import ConfigParser
from .imports import config_file
def get_path():
    cfg = ConfigParser()
    try:
        cfg.read_string(config_file)
    except:
        return r'{} '.format("vlc")
    if os.name=='nt':
        path = cfg.get("windows", "vlc")
    else:
        path = cfg.get("linux", "vlc")
    return r'{} '.format(path)

def play(videos):
    if os.name!='nt': subprocess.Popen(get_path()+ videos +" 2> /dev/null", shell=True)
    else: subprocess.Popen(get_path()+ videos, shell=True)


import sys, os
from .distrun import run
from .voice_recognition import voice
from .search import search
from .playmedia import play

def main():
    if sys.argv[0][-4:]=='.exe': # For Windows .exe file!
        run()

    elif len(sys.argv)>1:   # For linux
        music_name = " ".join(sys.argv[1:])
    else:
        try:
            music_name=voice()
        except KeyboardInterrupt:
            if os.name=='nt': print('See you again!') 
            else: print('ee you again!')
            sys.exit(0)
    if music_name is not None:
        videos = search(music_name)
        play(videos)

if __name__=="__main__":
    main()


import re, urllib.parse, urllib.request
from .commands import get_list

def get_link(music_name):
    query= urllib.parse.urlencode({"search_query": music_name.rstrip().lstrip()})
    searchUrl = urllib.request.urlopen("https://www.youtube.com/results?" + query)
    search_results = re.findall(r"watch\?v=(\S{11})", searchUrl.read().decode())
    return "https://www.youtube.com/watch?v=" + "{}".format(search_results[0])

def search(query):
    queries=get_list(query)
    links = ""
    for query in queries:
        if query == "": continue
        links+=get_link(query)+" "
    return links



def get_list(query):
    queries=""
    if ',' in query:
        queries = query.split(',')
    elif 'plus' in query:
        queries = query.split('plus')
    elif '+' in query:
        queries = query.split('+')
    elif 'then' in query:
        queries = query.split('then')
    elif 'than' in query:
        queries = query.split('than')
    elif 'den' in query:
        queries = query.split('den')    
    else:
        queries = [query]
    
    return queries


# import pathlib
from importlib import resources

# HERE = pathlib.Path(__file__).parent
# ascii_art = open(HERE / "ascii_art.txt", encoding="utf8")
# config_file = (HERE / "config.txt").read_text()

ascii_art = resources.read_text("youtube_stream", "ascii_art.txt")
config_file = resources.read_text("youtube_stream", "config.txt")




import speech_recognition as sr
import os
from .mute_shell import mute_on, mute_off
def recognize_speech_from_mic(recognizer, microphone):
    with microphone as source:
        recognizer.adjust_for_ambient_noise(source)
        audio = recognizer.listen(source)

    response = {
        "success": True,
        "error": None,
        "transcription": None
    }

    try:
        response["transcription"] = recognizer.recognize_google(audio)
    except sr.RequestError:
        response["success"] = False
        response["error"] = "API unavailable"
    except sr.UnknownValueError:
        response["error"] = "Unable to recognize speech"

    return response

def voice():

    recognizer = sr.Recognizer()
    if os.name!='nt': mute_on()
    microphone = sr.Microphone()
    if os.name!='nt': mute_off()
    print('Listening... (Press "Ctrl+C" to cancel)')
    voice_text = recognize_speech_from_mic(recognizer, microphone)
    if voice_text["transcription"]:
        print(voice_text['transcription'])
        return voice_text['transcription']

    if not voice_text["success"]:
        print("I didn't catch that. What did you say?\n")
    if voice_text["error"]:
        print("ERROR: {}".format(voice_text["error"]))
    


import pathlib
from setuptools import find_packages, setup
from youtube_stream.__init__ import __version__

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

setup(
    name="youtube-stream",
    version=__version__,
    description="Stream Youtube Music Videos from Command Line!",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/sanjib-sen/youtube-stream",
    author="Sanjib Kumar Sen",
    author_email="sksenonline@gmail.com",
    license="MIT",
    keywords = ['youtube', 'streaming', 'vlc', 'video', 'audio', 'automation'],
    download_url = 'https://github.com/sanjib-sen/youtube-stream/archive/v_'+__version__+'.tar.gz',
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Intended Audience :: End Users/Desktop",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Topic :: Home Automation",
        "Topic :: Multimedia :: Video :: Display",

    ],
    packages=find_packages(exclude=("tests",)),
    package_dir={'youtube_stream': 'youtube_stream'},
    package_data={'youtube_stream': ['*.txt']},
    include_package_data=True,
    install_requires=["SpeechRecognition"],
    entry_points={
        "console_scripts": [
            "stream=youtube_stream.__main__:main",
        ]
    },
)